@@ -1,4 +1,4 @@
-// Copyright 2023 The NATS Authors
+// Copyright 2023-2025 The NATS Authors
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -424,6 +424,8 @@ func BenchmarkJetStreamConsumeWithFilters(b *testing.B) {
 	}{
 		{1, 1, nats.MemoryStorage},
 		{3, 3, nats.MemoryStorage},
+		{1, 1, nats.FileStorage},
+		{3, 3, nats.FileStorage},
 	}
 
 	benchmarksCases := []struct {
@@ -1760,6 +1762,9 @@ func startJSClusterAndConnect(b *testing.B, clusterSize int) (c *cluster, s *Ser
 		shutdown = func() {
 			s.Shutdown()
 		}
+		s.optsMu.Lock()
+		s.opts.SyncInterval = 5 * time.Minute
+		s.optsMu.Unlock()
 	} else {
 		c = createJetStreamClusterExplicit(b, "BENCH_PUB", clusterSize)
 		c.waitOnClusterReadyWithNumPeers(clusterSize)
@@ -1768,6 +1773,11 @@ func startJSClusterAndConnect(b *testing.B, clusterSize int) (c *cluster, s *Ser
 		shutdown = func() {
 			c.shutdown()
 		}
+		for _, s := range c.servers {
+			s.optsMu.Lock()
+			s.opts.SyncInterval = 5 * time.Minute
+			s.optsMu.Unlock()
+		}
 	}
 
 	nc, err = nats.Connect(s.ClientURL())
