@@ -1,4 +1,4 @@
-// Copyright 2022-2023 The NATS Authors
+// Copyright 2022-2025 The NATS Authors
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -1526,3 +1526,237 @@ func TestJetStreamConsumerBackoffWhenBackoffLengthIsEqualToMaxDeliverConfig(t *t
 	require_NoError(t, err)
 	require_LessThan(t, time.Since(firstMsgSent), calculateExpectedBackoff(3))
 }
+
+func TestJetStreamConsumerRetryAckAfterTimeout(t *testing.T) {
+	for _, ack := range []struct {
+		title  string
+		policy nats.SubOpt
+	}{
+		{title: "AckExplicit", policy: nats.AckExplicit()},
+		{title: "AckAll", policy: nats.AckAll()},
+	} {
+		t.Run(ack.title, func(t *testing.T) {
+			s := RunBasicJetStreamServer(t)
+			defer s.Shutdown()
+
+			nc, js := jsClientConnect(t, s)
+			defer nc.Close()
+
+			_, err := js.AddStream(&nats.StreamConfig{
+				Name:     "TEST",
+				Subjects: []string{"foo"},
+			})
+			require_NoError(t, err)
+
+			_, err = js.Publish("foo", nil)
+			require_NoError(t, err)
+
+			sub, err := js.PullSubscribe("foo", "CONSUMER", ack.policy)
+			require_NoError(t, err)
+
+			msgs, err := sub.Fetch(1)
+			require_NoError(t, err)
+			require_Len(t, len(msgs), 1)
+
+			msg := msgs[0]
+			// Send core request so the client is unaware of the ack being sent.
+			_, err = nc.Request(msg.Reply, nil, time.Second)
+			require_NoError(t, err)
+
+			// It could be we have already acked this specific message, but we haven't received the success response.
+			// Retrying the ack should not time out and still signal success.
+			err = msg.AckSync()
+			require_NoError(t, err)
+		})
+	}
+}
+
+func TestJetStreamConsumerSwitchLeaderDuringInflightAck(t *testing.T) {
+	s := RunBasicJetStreamServer(t)
+	defer s.Shutdown()
+
+	nc, js := jsClientConnect(t, s)
+	defer nc.Close()
+
+	_, err := js.AddStream(&nats.StreamConfig{
+		Name:     "TEST",
+		Subjects: []string{"foo"},
+	})
+	require_NoError(t, err)
+
+	for i := 0; i < 2_000; i++ {
+		_, err = js.Publish("foo", nil)
+		require_NoError(t, err)
+	}
+
+	sub, err := js.PullSubscribe(
+		"foo",
+		"CONSUMER",
+		nats.MaxAckPending(2_000),
+		nats.ManualAck(),
+		nats.AckExplicit(),
+		nats.AckWait(2*time.Second),
+	)
+	require_NoError(t, err)
+
+	acc, err := s.lookupAccount(globalAccountName)
+	require_NoError(t, err)
+	mset, err := acc.lookupStream("TEST")
+	require_NoError(t, err)
+	o := mset.lookupConsumer("CONSUMER")
+	require_NotNil(t, o)
+
+	msgs, err := sub.Fetch(2_000)
+	require_NoError(t, err)
+	require_Len(t, len(msgs), 2_000)
+
+	// Simulate an ack being pushed, and o.setLeader(false) being called before the ack is processed and resets o.awl
+	atomic.AddInt64(&o.awl, 1)
+	o.setLeader(false)
+	o.setLeader(true)
+
+	msgs, err = sub.Fetch(1, nats.MaxWait(5*time.Second))
+	require_NoError(t, err)
+	require_Len(t, len(msgs), 1)
+}
+
+func TestJetStreamConsumerMessageDeletedDuringRedelivery(t *testing.T) {
+	storageTypes := []nats.StorageType{nats.MemoryStorage, nats.FileStorage}
+	for _, storageType := range storageTypes {
+		t.Run(storageType.String(), func(t *testing.T) {
+			s := RunBasicJetStreamServer(t)
+			defer s.Shutdown()
+
+			nc, js := jsClientConnect(t, s)
+			defer nc.Close()
+
+			_, err := js.AddStream(&nats.StreamConfig{
+				Name:     "TEST",
+				Subjects: []string{"foo"},
+				Storage:  storageType,
+			})
+			require_NoError(t, err)
+
+			for i := 0; i < 3; i++ {
+				_, err = js.Publish("foo", nil)
+				require_NoError(t, err)
+			}
+
+			sub, err := js.PullSubscribe(
+				"foo",
+				"CONSUMER",
+				nats.ManualAck(),
+				nats.AckExplicit(),
+				nats.AckWait(time.Second),
+			)
+			require_NoError(t, err)
+
+			acc, err := s.lookupAccount(globalAccountName)
+			require_NoError(t, err)
+			mset, err := acc.lookupStream("TEST")
+			require_NoError(t, err)
+			o := mset.lookupConsumer("CONSUMER")
+			require_NotNil(t, o)
+
+			msgs, err := sub.Fetch(3)
+			require_NoError(t, err)
+			require_Len(t, len(msgs), 3)
+
+			err = js.DeleteMsg("TEST", 2)
+			require_NoError(t, err)
+
+			// Wait for mset.storeUpdates to call into o.decStreamPending which runs
+			// the o.processTerm goroutine, removing one message from pending.
+			checkFor(t, 2*time.Second, 100*time.Millisecond, func() error {
+				o.mu.RLock()
+				defer o.mu.RUnlock()
+				if len(o.pending) != 2 {
+					return fmt.Errorf("expected 2 pending, but got %d", len(o.pending))
+				}
+				return nil
+			})
+
+			// Now empty the redelivery queue and reset the pending state.
+			o.mu.Lock()
+			for _, seq := range o.rdq {
+				o.removeFromRedeliverQueue(seq)
+			}
+			o.pending = make(map[uint64]*Pending)
+			o.pending[2] = &Pending{}
+			o.addToRedeliverQueue(2)
+
+			// Also reset delivery/ack floors to confirm they get corrected.
+			o.adflr, o.asflr = 0, 0
+			o.dseq, o.sseq = 11, 11
+
+			// Getting the next message should skip seq 2, as that's deleted, but must not touch state.
+			_, _, err = o.getNextMsg()
+			o.mu.Unlock()
+			require_Error(t, err, ErrStoreEOF)
+			require_Len(t, len(o.pending), 1)
+
+			// Simulate the o.processTerm goroutine running after a call to o.getNextMsg.
+			// Pending state and delivery/ack floors should be corrected.
+			o.processTerm(2, 2, 1, ackTermUnackedLimitsReason, _EMPTY_)
+
+			o.mu.RLock()
+			defer o.mu.RUnlock()
+			require_Len(t, len(o.pending), 0)
+			require_Equal(t, o.adflr, 10)
+			require_Equal(t, o.asflr, 10)
+		})
+	}
+}
+
+func TestJetStreamConsumerDeliveryCount(t *testing.T) {
+	s := RunBasicJetStreamServer(t)
+	defer s.Shutdown()
+
+	nc, js := jsClientConnect(t, s)
+	defer nc.Close()
+
+	_, err := js.AddStream(&nats.StreamConfig{
+		Name:     "TEST",
+		Subjects: []string{"foo"},
+	})
+	require_NoError(t, err)
+
+	for i := 0; i < 2; i++ {
+		_, err = js.Publish("foo", nil)
+		require_NoError(t, err)
+	}
+
+	sub, err := js.PullSubscribe(
+		"foo",
+		"CONSUMER",
+		nats.ManualAck(),
+		nats.AckExplicit(),
+		nats.AckWait(time.Second),
+		nats.MaxDeliver(1),
+	)
+	require_NoError(t, err)
+
+	acc, err := s.lookupAccount(globalAccountName)
+	require_NoError(t, err)
+	mset, err := acc.lookupStream("TEST")
+	require_NoError(t, err)
+	o := mset.lookupConsumer("CONSUMER")
+	require_NotNil(t, o)
+
+	msgs, err := sub.Fetch(2)
+	require_NoError(t, err)
+	require_Len(t, len(msgs), 2)
+	require_NoError(t, msgs[1].Nak())
+
+	require_Equal(t, o.deliveryCount(1), 1)
+	require_Equal(t, o.deliveryCount(2), 1)
+
+	// max deliver 1 so this will fail
+	_, err = sub.Fetch(1, nats.MaxWait(250*time.Millisecond))
+	require_Error(t, err)
+
+	// This would previously report delivery count 0, because o.rdc!=nil
+	require_Equal(t, o.deliveryCount(1), 1)
+	require_Equal(t, o.deliveryCount(2), 1)
+
+}
