@@ -1,4 +1,4 @@
-// Copyright 2015-2020 The NATS Authors
+// Copyright 2015-2024 The NATS Authors
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
 // You may obtain a copy of the License at
@@ -66,7 +66,7 @@ func TestTLSConnection(t *testing.T) {
 
 	nc.Publish(subj, []byte("We are Secure!"))
 	nc.Flush()
-	nmsgs, _ := sub.QueuedMsgs()
+	nmsgs, _, _ := sub.Pending()
 	if nmsgs != 1 {
 		t.Fatalf("Expected to receive a message over the TLS connection")
 	}
@@ -1100,7 +1100,7 @@ func TestTLSConnectionCurvePref(t *testing.T) {
 
 	nc.Publish(subj, []byte("We are Secure!"))
 	nc.Flush()
-	nmsgs, _ := sub.QueuedMsgs()
+	nmsgs, _, _ := sub.Pending()
 	if nmsgs != 1 {
 		t.Fatalf("Expected to receive a message over the TLS connection")
 	}
